; Project: PicKey 
; Program name: PicKey.asm 
; Author: Gabriel Rapetti
; Date: 01/10/2012
; Micro: PIC12F629
; Clock: internal 4MHz
; ----------------------------------------------------------------------------------------------

		ERRORLEVEL -302	   
  		LIST 	P = 12F629, r=DEC, n=66
    	INCLUDE         "P12F629.INC"
		
		__CONFIG _CP_ON & _CPD_OFF & _BODEN_ON & _MCLRE_OFF & _WDT_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT

;-----------------------------------------------------------------------------------------------

;-----------------------------------------------------------------------------------------------
;					CONSTANT
;-----------------------------------------------------------------------------------------------

keycode1	equ		23			; KEY CODE 1
keycode2	equ		78			; KEY CODE 2

BAUD_1		equ    .30 			; 3+3X = CLKOUT/Baud
BAUD_X		equ    .30			; 11+3X = CLKOUT/Baud

;-----------------------------------------------------------------------------------------------------------
; Pin In /Out map
; =================
;-----------------------------------------------------------------------------------------------------------

#define		p_tx	GPIO,0			; 

;-----------------------------------------------------------------------------------------------
;					REGISTER IN RAM
;-----------------------------------------------------------------------------------------------
	cblock     20H     ; Dummy Origin

; Serial registers
;---------------------------------------
		TxReg				; Serial Tx register
		BitCount			; Tx bits counter 

; Delay register
;---------------------------------------
		CountDelay1			; time delay
		CountDelay2			; long time delay 2
	endc


;-----------------------------------------------------------------------------------------------------------
; MACRO Pag change in RAM
; ==========================
;-----------------------------------------------------------------------------------------------------------

PAG_RAM0	MACRO
			bcf	STATUS,RP0
			ENDM

PAG_RAM1	MACRO
			bsf	STATUS,RP0
			ENDM


;-----------------------------------------------------------------------------------------------------------
; Start Vectors
; ==============
;-----------------------------------------------------------------------------------------------------------

    		ORG     0
			PAG_RAM1
			call 	3FFh 			;Get the cal value
			movwf 	OSCCAL 			;Calibrate
			PAG_RAM0
			goto	main

;-----------------------------------------------------------------------------------------------------------
;					SUBRUTINES
;-----------------------------------------------------------------------------------------------------------
			ORG	10h					; beginning of program Eprom

;-----------------------------------------------------------------------------------------------------------
; Tx
; ====
;-----------------------------------------------------------------------------------------------------------
Tx
		clrwdt
		bsf		p_tx
		movlw   8				; load bit counter with 8
		movwf   BitCount		; 

		bcf     p_tx			; pin_tx to low (start bit)
		call    Delay1			; delay start bit 
X1_next
		bcf     STATUS,C		; clear status bit
		rrf     TxReg,F			; load next bit in the Status C register 

		btfsc   STATUS,C		; bit is Hight?
		bsf     p_tx			; Yes, set pin_tx
		btfss   STATUS,C		; bit is Low?
		bcf     p_tx			; Yes, reset pin_tx
      
		call    DelayX			; delay inter_bit
		decfsz  BitCount,F		; all 8 bit Tx?
		goto    X1_next			; No, jump to next bit

		bsf     p_tx			; Yes, pin_tx to high (stop bit)
		call    Delay1			; delay stop bit 
		retlw   0				; End

;-----------------------------------------------------------------------------------------------------------
; delay Tx / Rx 
; ================
;-----------------------------------------------------------------------------------------------------------

DelayX
		movlw   BAUD_X			; Serial TX Delay (interbit)
		goto    save
Delay1
		movlw   BAUD_1			; Serial TX Delay (start e stop)
		goto    save
save
		movwf   CountDelay1
redo_1
	  	decfsz  CountDelay1,F
		goto    redo_1
		retlw   0

;-----------------------------------------------------------------------------------------------------------
; LongDelay
; ================
;-----------------------------------------------------------------------------------------------------------
LongDelay		
		movlw	0FFH			; X2 multiplicator
		movwf	CountDelay2

DlyJp1	clrwdt
		movlw	0FFH			; X1 multiplicator
		movwf	CountDelay1
DlyJp2	decfsz	CountDelay1,F
		goto	DlyJp2
		decfsz	CountDelay2,F
		goto 	DlyJp1
		retlw	0

;-----------------------------------------------------------------------------------------------
; 					MAIN 
;-----------------------------------------------------------------------------------------------

main    	
		PAG_RAM1
		movlw   B'00000000'    		; GP1 (mode) and GP4 (key) = input 
 		movwf	TRISIO       		; 

		movlw	B'01101111'			; Pull-up off. Prscaler %128 in WDT	
		movwf	OPTION_REG			; 

		PAG_RAM0		

		movlw	B'00000111'			; Set port as digital input
		movwf	CMCON

		bcf	INTCON,GIE				; global interrupt off
		bcf	INTCON,PEIE				; periferical interrupt off
;-------------------------------------------------------------------------------------------- 
; ciclo principal
; ================
;-------------------------------------------------------------------------------------------- 
MainLoop
		call	LongDelay			; Start Delay
		movlw	keycode1			; Load keycode 1
		movwf	TxReg				; Tx code
		call	Tx
		call	DelayX				; InterByte delay
		movlw	keycode2			; Load keycode2
		movwf	TxReg				; Tx code
		call	Tx				
		goto	MainLoop			; jump to loop

;***********************************************************************************************
; Service Interrupt: SELECT
;***********************************************************************************************
service_int
		retfie
;***********************************************************************************************
		END
