; Project: PicKey
; Program name: PicKeyBase.asm
; Author: Gabriel Rapetti
; Date: 01/10/2012
; Micro: PIC12F629
; Clock: internal 4MHz
; ----------------------------------------------------------------------------------------------

		ERRORLEVEL -302	   
  		LIST 	P = 12F629, r=DEC, n=66
    	INCLUDE         "P12F629.INC"
		
		__CONFIG _CP_ON & _CPD_OFF & _BODEN_ON & _MCLRE_OFF & _WDT_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT

;-----------------------------------------------------------------------------------------------

;-----------------------------------------------------------------------------------------------
;					CONSTANT
;-----------------------------------------------------------------------------------------------

keycode1	equ		23			; KEY CODE 1
keycode2	equ		78			; KEY CODE 2

BAUD_4		equ    .41			; 3+3X = 1.25*CLKOUT/Baud
BAUD_Y		equ    .29			; 9 +3X = CLKOUT/Baud

;-----------------------------------------------------------------------------------------------------------
; Pin In /Out map
; =================
;-----------------------------------------------------------------------------------------------------------

#define		p_rx	GPIO,4		; Key Rx pin 
#define		p_mode	GPIO,1		; mode 		
#define		p_rele	GPIO,2		; relay output

;-----------------------------------------------------------------------------------------------
;					REGISTER IN RAM
;-----------------------------------------------------------------------------------------------
		cblock     20H     		; Dummy Origin

; Serial registers
;---------------------------------------
			RxReg				; Rx register
			BitCount			; Rx bits counter 

; Delay register
;---------------------------------------
			CountDelay1			; time delay 1
			CountDelay2			; time delay 2
			CountDelay3			; time delay 3
			TimeOutReg			; Time Out second byte rx
		endc


;-----------------------------------------------------------------------------------------------------------
; MACRO Pag change in RAM
; ==========================
;-----------------------------------------------------------------------------------------------------------

PAG_RAM0	MACRO
			bcf	STATUS,RP0
			ENDM

PAG_RAM1	MACRO
			bsf	STATUS,RP0
			ENDM


;-----------------------------------------------------------------------------------------------------------
; Start Vectors
; ==============
;-----------------------------------------------------------------------------------------------------------

    		ORG     0
			PAG_RAM1
			call 	3FFh 			;Get the cal value
			movwf 	OSCCAL 			;Calibrate
			PAG_RAM0
			goto	main

;-----------------------------------------------------------------------------------------------------------
;					SUBRUTINES
;-----------------------------------------------------------------------------------------------------------
			ORG	10h					; beginning of program Eprom

;-----------------------------------------------------------------------------------------------------------
; Rx
; ====
;-----------------------------------------------------------------------------------------------------------

Rx
		clrwdt
		clrf    RxReg			; 
		call    Delay4			; delay to check in center of the start bit
		movlw   8				; load bit counter with 8 bits 
		movwf   BitCount		; 
R1_next
		bcf     STATUS,C		; clear Status C reg
		rrf     RxReg,F			; one bit shift RxReg

		btfsc   p_rx			; check Rx pin
		bsf     RxReg,7			; Rx pins is Hight? Yes, set bit on the RxReg 

		call    DelayY			; delay inter_bit
		decfsz  BitCount,F		; all 8 bit Rx?
		goto    R1_next			; No, jump to next bit
		retlw   0				; End


;-----------------------------------------------------------------------------------------------------------
; delay Tx / Rx 
; ================
;-----------------------------------------------------------------------------------------------------------
DelayY
		movlw   BAUD_Y			; Serial RX Delay 1 bit (interbit)	
		goto    save
Delay4
		movlw   BAUD_4			; Serial RX Delay di 1+1/2 bit (Start + 1/2 bit) 
		goto    save
save
		movwf   CountDelay1
redo_1
	  	decfsz  CountDelay1,F
		goto    redo_1
		retlw   0

;-----------------------------------------------------------------------------------------------------------
; delay release (wait to key realease)
; ====================================
;-----------------------------------------------------------------------------------------------------------
DlyRelease	
		movlw	70				; X3 multiplicator
		movwf	CountDelay3
DlyJp0
		movlw	70				; X2 multiplicator
		movwf	CountDelay2

DlyJp1	clrwdt
		movlw	70				; X1 multiplicator
		movwf	CountDelay1

DlyJp2	btfss	p_rx			
		goto	DlyRelease
		decfsz	CountDelay1,F
		goto	DlyJp2

		decfsz	CountDelay2,F
		goto 	DlyJp1

		decfsz	CountDelay3,F
		goto 	DlyJp0

		retlw	0

;-----------------------------------------------------------------------------------------------------------
; Relay Toggle
; ===============
;-----------------------------------------------------------------------------------------------------------
RelayToggle
		btfss	p_rele				; relay is off?
		goto	ReleOn				; Yes, then jump to relay On 
		bcf		p_rele				; No, then relay off
		retlw	0					; return

ReleOn
		bsf		p_rele				; relay On
		retlw	0					; return

;-----------------------------------------------------------------------------------------------
; 					MAIN 
;-----------------------------------------------------------------------------------------------

main    	
		PAG_RAM1
		movlw   B'00010010'    		; GP1 (mode) and GP4 (key) = input 
 		movwf	TRISIO       		; 

		movlw	B'00010010'			; individual Pull-up on
		movwf	WPU	

		movlw	B'01101111'			; Pull-up on. Prscaler %128 in WDT	
		movwf	OPTION_REG			; 

		PAG_RAM0		

		movlw	B'00000111'			; Set port as digital input
		movwf	CMCON

		bcf	INTCON,GIE				; global interrupt off
		bcf	INTCON,PEIE				; periferical interrupt off

		bsf		p_rele				; Relay On
		call	DlyRelease			; Delay
		bcf		p_rele				; Rel Off
 
;-------------------------------------------------------------------------------------------- 
; Main Loop
; ================
;-------------------------------------------------------------------------------------------- 
MainLoop		
		clrwdt						; Reset watchdog timer
		btfss	p_mode				; Toggle mode?
		bcf		p_rele				; No, then rel off
		btfsc	p_rx				; Key present?
		goto	MainLoop			; No, then jump to main loop

; -------------------------------------------------------------------------------------------		
; Key present 
; ===========		
; -------------------------------------------------------------------------------------------	
		
RxByte1
		call	Rx					; Key present, read byte 1
		movlw	keycode1			; compare rx byte 1 with local keycode1 
		subwf	RxReg,w				
		btfss	STATUS,Z			; different?
		goto	MainLoop			; Yes, then jump to main loop
		clrf	TimeOutReg			; Reset timeout interbyte counter 

JmpRxByte1
		clrwdt						; Reset watchdog timer
		btfss	p_rx				; second byte ready?	
		goto	RxByte2				; 
		decfsz	TimeOutReg,f		; No, decrement interbyte counter 
		goto	JmpRxByte1			; interbyte counter = 0? No, then jump wite second byte   
		goto	MainLoop			; Yes (timeout), then jump to main loop 

RxByte2
		call	Rx					; read byte 2	
		movlw	keycode2			; compare rx byte 2 with local keycode2 
		subwf	RxReg,w	
		btfss	STATUS,Z			; different?
		goto	MainLoop			; Yes, then jump to main loop

		btfsc	p_mode				; toggle mode?
		goto	FlipFlopMode		; Yes, jump to toggle mode

		bsf		p_rele				; relay ON
		call	DlyRelease			; wait to key realease
		bcf		p_rele				; relay OFF 
		goto	MainLoop			; jump to main loop 
; -------------------------------------------------------------------------------------------		
; Relay toggle mode 
; ===================		
; -------------------------------------------------------------------------------------------	
FlipFlopMode
		call	RelayToggle			; relay On / Off (toggle)
		call	DlyRelease			; wait to key realease	
		goto	MainLoop			; jump to main loop

;***********************************************************************************************
; Service Interrupt: SELECT
;***********************************************************************************************
service_int
		retfie
;***********************************************************************************************
		END
